module RedmineVersionFields
  # Patches Redmine's Query dynamically to include the two new fields in the filter list.
  module QueryPatch

    def self.included(base) # :nodoc:
      base.send(:include, InstanceMethods)
      base.alias_method_chain :available_filters, :wrapping
    end

    # This module wraps the original available_filters method, adding two new 
    # filters to the returned list.
    module InstanceMethods

      def available_filters_with_wrapping
        # Generate the original list, by calling the original function
        @available_filters = available_filters_without_wrapping

        # If we have a project, grab all available versions and use them to build
        # filter options for found in and release notes target
        if project
          # Add Found in and Release notes target to filter list.
          unless @project.shared_versions.empty?
            @available_filters["found_in_version_id"] = { :type => :list_optional, :order => 16, :values => @project.shared_versions.sort.collect{|s| ["#{s.project.name} - #{s.name}", s.id.to_s] } }
            @available_filters["release_notes_version_id"] = { :type => :list_optional, :order => 17, :values => @project.shared_versions.sort.collect{|s| ["#{s.project.name} - #{s.name}", s.id.to_s] } }
          end
        end
        @available_filters
      end
    end

  end
end
